package com.bwie.auth.service.impl;

import cn.hutool.core.util.RandomUtil;
import com.bwie.auth.service.AuthRegService;
import com.bwie.auth.util.MsgUtil;
import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.RegRequest;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.remote.system.RemoteUserService;
import com.bwie.common.result.Result;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.servlet.http.HttpServletRequest;
import java.util.concurrent.TimeUnit;

@Service
public class AuthRegServiceImpl implements AuthRegService {
    private final RedisCache redisCache;
    private final RemoteUserService remoteUserService;
    private final HttpServletRequest request;
    private final MsgUtil msgUtil;

    public AuthRegServiceImpl(RedisCache redisCache, RemoteUserService remoteUserService, HttpServletRequest request, MsgUtil msgUtil) {
        this.redisCache = redisCache;
        this.remoteUserService = remoteUserService;
        this.request = request;
        this.msgUtil = msgUtil;
    }

    @Override
    public void sendcode(String userTel) {
        String s = RandomUtil.randomNumbers(4);
        redisCache.setCacheObject("code:"+userTel,s,Long.valueOf(5), TimeUnit.MINUTES);
        msgUtil.sendMsg(userTel,s);
    }

    @Override
    public void reg(RegRequest request) {
        Result<User> findtel = remoteUserService.findtel(request.getUserTel());
        Assert.isTrue(findtel.isSuccess(),"注册失败");
        User data = findtel.getData();
        Assert.isNull(data,"电话已存在");

        String cacheObject = redisCache.getCacheObject("code:" + request.getUserTel());
        Assert.isTrue(cacheObject.equals(request.getUserCode()),"验证码错误");
        Result<String> add=remoteUserService.adduser(
                RegRequest.builder()
                        .userName(request.getUserName())
                        .userPwd(request.getUserPwd())
                        .userSex(request.getUserSex())
                        .userAddr(request.getUserAddr())
                        .userBirth(request.getUserBirth())
                        .userTel(request.getUserTel())
                        .build()
        );
        Assert.isTrue(add.isSuccess(),"注册失败");

    }

    @Override
    public User getInfo() {
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + request.getHeader(JwtConstants.USER_KEY));
        return  cacheObject;

    }
}
