package com.bwie.auth.service.impl;





import cn.hutool.http.server.HttpServerRequest;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LoginRequest;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.remote.system.RemoteUserService;
import com.bwie.common.result.Result;
import com.bwie.common.utils.IdUtils;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.SecurityUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;


import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@Service
public class AuthServiceImpl implements AuthService {
    private final RedisCache redisCache;
    private final RemoteUserService remoteUserService;


    public AuthServiceImpl(RedisCache redisCache, RemoteUserService remoteUserService) {
        this.redisCache = redisCache;
        this.remoteUserService = remoteUserService;

    }

    @Override
    public JwtResponse login(LoginRequest loginRequest) {
        Result<User> findname = remoteUserService.findname(loginRequest.getUserName());
        Assert.isTrue(findname.isSuccess(),"用户登陆失败");
        User data = findname.getData();
        Assert.notNull(data,"用户不存在");
        Assert.isTrue(
                SecurityUtils.matchesPassword(
                        loginRequest.getUserPwd(),data.getSalt(),data.getUserPwd()
                ),"用户或密码错误"
        );
        HashMap<String, Object> map = new HashMap<>();
        String userkey = IdUtils.genId();
        map.put(JwtConstants.USER_KEY,userkey);
        map.put(JwtConstants.DETAILS_USER_ID,data.getUserId());
        map.put(JwtConstants.DETAILS_USERNAME,data.getUserName());
        String token = JwtUtils.createToken(map);
        redisCache.setCacheObject(TokenConstants.LOGIN_TOKEN_KEY+userkey
        ,data,TokenConstants.EXPIRATION, TimeUnit.SECONDS);

        Result<List<User>> users=remoteUserService.findfriends(data.getUserId());

        for (User user : users.getData()) {
            redisCache.setCacheObject("用户:"+data.getUserId(),user);
        }

        remoteUserService.updtime(data.getUserId());

        return JwtResponse.builder()
                .token(token)
                .expirTime(TokenConstants.EXPIRATION)
                .build();
    }
}
