package com.bwie.common.remote.system.factory;

import com.bwie.common.domain.Red;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.RegRequest;
import com.bwie.common.domain.request.SendredRequest;
import com.bwie.common.remote.system.RemoteUserService;
import com.bwie.common.result.Result;
import lombok.extern.log4j.Log4j2;
import org.springframework.cloud.openfeign.FallbackFactory;
import org.springframework.stereotype.Component;

import java.util.List;

@Log4j2
@Component
public class RemoteUserFactory implements FallbackFactory<RemoteUserService> {
    @Override
    public RemoteUserService create(Throwable cause) {
        return new RemoteUserService() {
            @Override
            public Result<User> findname(String userName) {
                log.error("查找用户远程调用失败:[{}-{}]",userName,cause.getMessage(),cause);
                return Result.error();
            }

            @Override
            public Result<User> findtel(String userTel) {
                log.error("查找电话远程调用失败,[{}-{}]",userTel,cause.getMessage(),cause);
                return Result.error();
            }

            @Override
            public Result<String> adduser(RegRequest build) {
                log.error("添加用户远程调用失败,[{}-{}]",build,cause.getMessage(),cause);
                return Result.error();
            }

            @Override
            public Result<List<User>> findfriends(Long userId) {
                log.error("查找朋友远程调用失败,[{}-{}]",userId,cause.getMessage(),cause);
                return Result.error();
            }

            @Override
            public Result updmoney(SendredRequest sendredRequest) {
                log.error("修改余额远程调用失败,[{}-{}]",sendredRequest,cause.getMessage(),cause);
                return Result.error();
            }

            @Override
            public Result backmoney(List<Red> findred) {
                log.error("修改余额远程调用失败,[{}-{}]",findred,cause.getMessage(),cause);
                return Result.error();
            }

            @Override
            public Result updtime(Long userId) {
                log.error("修改登陆状态,时间远程调用失败,[{}-{}]",userId,cause.getMessage(),cause);
                return Result.error();
            }

            @Override
            public Result addmoney(Long userId, Double increMoney) {
                log.error("抢到红包增加余额远程调用失败,[{}-{}]",userId,cause.getMessage(),cause);
                return Result.error();
            }
        };
    }
}
