package com.bwie.friend.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.Friend;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.DelRequest;
import com.bwie.common.domain.request.FriendRequest;
import com.bwie.common.domain.request.ShowRequest;
import com.bwie.common.domain.response.Users;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.remote.system.RemoteUserService;
import com.bwie.common.result.Result;
import com.bwie.friend.mapper.FriendMapper;
import com.bwie.friend.service.FriendService;
import com.bwie.friend.util.MsgUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Service
@Log4j2
public class FriendServiceImpl extends ServiceImpl<FriendMapper, Friend> implements FriendService {
    private final FriendMapper friendMapper;
    private final HttpServletRequest request;
    private final RemoteUserService remoteUserService;
    private final MsgUtil msgUtil;
    private final RedisCache redisCache;


    public FriendServiceImpl(FriendMapper friendMapper, HttpServletRequest request, RemoteUserService remoteUserService, MsgUtil msgUtil, RedisCache redisCache) {
        this.friendMapper = friendMapper;
        this.request = request;
        this.remoteUserService = remoteUserService;
        this.msgUtil = msgUtil;
        this.redisCache = redisCache;

    }

    @Override
    public Result show(FriendRequest friendRequest) {
        Long userId = getInfo().getUserId();
        friendRequest.setUserId(userId);;
        List<User> show = friendMapper.show(friendRequest);
        return Result.success(show);
    }

    @Override
    public Result showfriend() {
        Long userId = getInfo().getUserId();
        List<Users> showfriend = friendMapper.showfriend(userId);
        return Result.success(showfriend);
    }

    @Override
    public Result delfriend(DelRequest delRequest) {

        friendMapper.delfriend(delRequest.getUserFr(),getInfo().getUserId());
        log.info("您好！您的好友[{}]，在[{}]把您删除",getInfo().getUserName(),new Date().toLocaleString());
        msgUtil.sendMsg(delRequest.getUserTel(),"1234");
        return Result.success("删除成功");
    }

    @Override
    public Result showall(ShowRequest showRequest) {
        showRequest.setUserId(getInfo().getUserId());
        List<User> showall = friendMapper.showall(showRequest);
        return Result.success(showall);
    }

    @Override
    public Result addfriend() {
        redisCache.deleteObject("用户:"+getInfo().getUserId());

        Result<List<User>> findfriends = remoteUserService.findfriends(getInfo().getUserId());

        for (User user : findfriends.getData()) {
            redisCache.setCacheObject("用户:"+user.getUserId(),user);
        }
        return Result.success("添加成功");
    }

    public  User getInfo() {
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + request.getHeader(JwtConstants.USER_KEY));
        return  cacheObject;
    }
}
