package com.bwie.log.service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bwie.common.domain.Log;
import com.bwie.log.mapper.LogMapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.common.unit.Fuzziness;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.elasticsearch.search.sort.FieldSortBuilder;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class LogServiceImpl extends ServiceImpl<LogMapper,Log> implements LogService {

   private final RestHighLevelClient client;

    public LogServiceImpl(RestHighLevelClient client) {
        this.client = client;
    }

    @Override
    public void addbach(List<Log> logs) {
        BulkRequest bulkRequest = new BulkRequest();

        if(logs.size()>0){
            for (Log log : logs) {
                ObjectMapper objectMapper = new ObjectMapper();
                try {
                    String s = objectMapper.writeValueAsString(log);
                    bulkRequest.add(new IndexRequest("log").id(log.getLogId().toString())
                            .source(s, XContentType.JSON));

                } catch (JsonProcessingException e) {
                    throw new RuntimeException(e);
                }
            }
            try {
                client.bulk(bulkRequest, RequestOptions.DEFAULT);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

    }

    @Override
    public List<Log> showlog(Long redId) {
        SearchRequest searchRequest = new SearchRequest("log");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        BoolQueryBuilder boolQueryBuilder = new BoolQueryBuilder();

        if(null!=redId&&!redId.equals("")){
            boolQueryBuilder.must(QueryBuilders.termQuery("redId",redId));
        }


        searchSourceBuilder.query(boolQueryBuilder);



        searchRequest.source(searchSourceBuilder);
        ArrayList<Log> logs = new ArrayList<>();
        try {
            SearchResponse search = client.search(searchRequest, RequestOptions.DEFAULT);

            SearchHits hits = search.getHits();
            for (SearchHit hit : hits) {


                String sourceAsString = hit.getSourceAsString();
                ObjectMapper objectMapper = new ObjectMapper();
                Log log1 = objectMapper.readValue(sourceAsString, Log.class);

               logs.add(log1);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return logs;
    }
}
