package com.bwie.red.service.impl;

import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import com.bwie.common.domain.Log;
import com.bwie.common.domain.Red;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.RedRequest;
import com.bwie.common.domain.request.SendredRequest;
import com.bwie.common.domain.request.SnatchingQuest;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.remote.log.RemoteLogService;
import com.bwie.common.remote.system.RemoteUserService;
import com.bwie.common.result.Result;

import com.bwie.red.mapper.RedMapper;
import com.bwie.red.service.RedService;
import com.bwie.red.util.MsgUtil;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Service
public class RedServiceImpl implements RedService {
    private final RedMapper redMapper;
    private final HttpServletRequest request;
    private final  RedissonClient redissonClient;

   private final RedisCache redisCache;
   private final RemoteUserService remoteUserService;
   private final RemoteLogService remoteLogService;
   private final MsgUtil msgUtil;


    public RedServiceImpl(RedMapper redMapper, HttpServletRequest request, RedissonClient redissonClient, RedisCache redisCache, RemoteUserService remoteUserService, RemoteLogService remoteLogService, MsgUtil msgUtil) {
        this.redMapper = redMapper;
        this.request = request;
        this.redissonClient = redissonClient;
        this.redisCache = redisCache;
        this.remoteUserService = remoteUserService;
        this.remoteLogService = remoteLogService;
        this.msgUtil = msgUtil;

    }


    @Override
    public Result show(RedRequest redRequest) {
        Long userId = getInfo().getUserId();
        redRequest.setUserId(userId);
        List<Red> show = redMapper.show(redRequest);
        return Result.success(show);
    }

    @Override
    public Result sendred(SendredRequest sendredRequest) {
        sendredRequest.setUserId(getInfo().getUserId());
        if(sendredRequest.getRedMoney()>300||sendredRequest.getRedMoney()>getInfo().getUserYu()){
            return Result.error("红包金额不可以大于300元，红包金额小于用户金额");
        }
        if(sendredRequest.getRedSum()>sendredRequest.getUsers().size()&&sendredRequest.getRedSum()<=0){
            return Result.error("红包个数错误");
        }
        if(sendredRequest.getRedMoney()/ sendredRequest.getRedSum()<=0.01){
            return Result.error("红包个数错误");
        }
        redMapper.addred(sendredRequest);
        remoteUserService.updmoney(sendredRequest);
        redMapper.addgrab(sendredRequest.getRedId(),sendredRequest.getUsers());
        return Result.success("发送成功");

    }

    @Override
    public Result snatching(SnatchingQuest snatchingQuest) {
        RLock one = redissonClient.getLock("one");
        try {
            one.tryLock(2, TimeUnit.MINUTES);
            if(snatchingQuest.getRedYu()/snatchingQuest.getRedNum()==0.01){
                snatchingQuest.setIncreMoney(0.01);
            }
            if (snatchingQuest.getRedYu()/snatchingQuest.getRedNum()>0.01) {
                snatchingQuest.setIncreMoney((snatchingQuest.getRedYu()/snatchingQuest.getRedNum())-0.01*(snatchingQuest.getRedNum()-1));
            }
//        snatchingQuest.setUserId(getInfo().getUserId());
            //修改红包余额 红包个数 是否抢到
            redMapper.updgrab(snatchingQuest);
            //修改余额 user表
            remoteUserService.addmoney(getInfo().getUserId(),snatchingQuest.getIncreMoney());
            //发送短信
            msgUtil.sendMsg(getInfo().getUserTel(),"1111");
            //添加日志记录表
            Log build = Log.builder().redId(snatchingQuest.getRedId())
                    .userName(getInfo().getUserName())
                    .redTheme(snatchingQuest.getRedTheme())
                    .logTime(new Date())
                    .build();
            remoteLogService.addlog(build);

            if(one.isHeldByCurrentThread()){
                if(one.isLocked()){
                    one.unlock();
                }
            }
            return Result.success("抢红包成功");

        } catch (InterruptedException e) {
            e.printStackTrace();
        }finally {
            if(one.isHeldByCurrentThread()){
                if(one.isLocked()){
                    one.unlock();
                }
            }
        }
        return Result.error("系统繁忙");



    }

    public  User getInfo() {
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + request.getHeader(JwtConstants.USER_KEY));
        return  cacheObject;
    }
}
